function [ output_args ] = polar_plot( acc_est1, acc_est2, acc_est3, acc_est4 )
%POLAR_PLOT Plot the slowly changes in forces in horizontal body plane
%   Input is estimated accelerations in one IMU, this is low pass filtered
%   to get the slowly varying changes, i.e. the drift in the accelerations.
%   These changes are plotted in a polar plot as a function of time

fig = figure('Units','centimeters','position',[0,0,20,30]);
F(50) = struct('cdata',[],'colormap',[]);
v = VideoWriter('T1,5.avi');
v.FrameRate = 48;
v.Quality = 100;
open(v);
num = 1;
for i = 1:length(acc_est1(1,:))
    %IMU1
    subplot(3,2,3);
    theta = [0 atan2(acc_est1(5,i),acc_est1(4,i))];
    rho = [0 sqrt(acc_est1(4,i)^2+acc_est1(5,i)^2)];
    polarplot(theta,rho, 'LineWidth',2);title('Port, aft');
    ax = gca;
    ax.ThetaLim = [-180 180];
    ax.ThetaDir = 'clockwise';
    ax.ThetaZeroLocation = 'top';
    ax.RLim = [0 400];
    
    %IMU2
    subplot(3,2,4);
    theta = [0 atan2(acc_est2(5,i),acc_est2(4,i))];
    rho = [0 sqrt(acc_est2(4,i)^2+acc_est2(5,i)^2)];
    polarplot(theta,rho, 'LineWidth',2);title('Starboard, aft');
    ax = gca;
    ax.ThetaLim = [-180 180];
    ax.ThetaDir = 'clockwise';
    ax.ThetaZeroLocation = 'top';
    ax.RLim = [0 400];
    
    %IMU3
    subplot(3,2,2);
    theta = [0 atan2(acc_est3(5,i),acc_est3(4,i))];
    rho = [0 sqrt(acc_est3(4,i)^2+acc_est3(5,i)^2)];
    polarplot(theta,rho, 'LineWidth',2);title('Starboard, bow');
    ax = gca;
    ax.ThetaLim = [-180 180];
    ax.ThetaDir = 'clockwise';
    ax.ThetaZeroLocation = 'top';
    ax.RLim = [0 400];
    
    %IMU4
    subplot(3,2,1);
    theta = [0 atan2(acc_est4(5,i),acc_est4(4,i))];
    rho = [0 sqrt(acc_est4(4,i)^2+acc_est4(5,i)^2)];
    polarplot(theta,rho, 'LineWidth',2);title('Port, bow');
    ax = gca;
    ax.ThetaLim = [-180 180];
    ax.ThetaDir = 'clockwise';
    ax.ThetaZeroLocation = 'top';
    ax.RLim = [0 400];
    
    %Heave acceleration
    subplot(3,2,[5 6]);
    bh = bar3([5, 7, 14, 16], [0 0 0 acc_est3(6,i); acc_est4(6,i) 0 0 0; 0 0 0 acc_est2(6,i); acc_est1(6,i) 0 0 0], 'r');title('Heave');
    bh(2).FaceColor = 'none';
    bh(2).EdgeColor = 'none';
    bh(3).FaceColor = 'none';
    bh(3).EdgeColor = 'none';
    hbar = gca;
    hbar.ZLim = [-600 600];
    pbaspect([1 3 1]);
    hbar.View = [-44 15];
    frame = getframe(gcf);
    while (i*0.05) > (num*0.0208)
        writeVideo(v,frame);
        num = num + 1;
    end
end
close(v);

end