function [ heading ] = wave_heading( acc_est1, acc_est2, acc_est3, acc_est4 )
%WAVE_HEADING Calculates heading of incoming waves
%   Based on how the wave propagates in the hull, the heading of incoming
%   waves is calculated, with a frequency of 2 pr period

vel = 2342;
heading = [];
j = 1;
k = 0;
switcher = 0;
time_corr = 120;
f = figure('Units','centimeters','Position',[0,0,15,8],'PaperPositionMode','auto');
plot(acc_est1(1,:)-time_corr,acc_est1(7,:),'r',acc_est2(1,:)-time_corr,acc_est2(7,:),'b',acc_est3(1,:)-time_corr,acc_est3(7,:),'m',acc_est4(1,:)-time_corr,acc_est4(7,:),'g');hold on
for i = 5:length(acc_est1(1,:))-10
    gradient = acc_est1(7,i)-acc_est1(7,i-4);
    
    %Search for local maximum
    if gradient < -200 && switcher == 1;
        [a, k] = max([acc_est1(7,i-14:i)]);
        num = i-15+k;
        [max1, index] = max([acc_est2(7,num-10:num+10);acc_est3(7,num-10:num+10);acc_est4(7,num-10:num+10)],[],2);
        index = [num-11; num-11; num-11]+index;
        t = acc_est1(1,num);
        heading(1,j) = t;
        measured_time = [acc_est2(1,index(1))-t, acc_est3(1,index(2))-t, acc_est4(1,index(3))-t];
        heading(2,j) = wave_dir(measured_time, vel);
        j = j+1;
        switcher = 0;
        plot(acc_est1(1,num)-time_corr,acc_est1(7,num),'ro',acc_est2(1,index(1))-time_corr,acc_est2(7,index(1)),'bo',...
            acc_est3(1,index(2))-time_corr,acc_est3(7,index(2)),'mo',acc_est4(1,index(3))-time_corr,acc_est4(7,index(3)),'go');
    
    %Local minima
    elseif gradient > 200 && switcher == 0
        [a, k] = min([acc_est1(7,i-14:i)]);
        num = i-15+k;
        [min1, index] = min([acc_est2(7,num-10:num+10);acc_est3(7,num-10:num+10);acc_est4(7,num-10:num+10)],[],2);
        index = [num-11; num-11; num-11]+index;
        t = acc_est1(1,num);
        heading(1,j) = t;
        measured_time = [acc_est2(1,index(1))-t, acc_est3(1,index(2))-t, acc_est4(1,index(3))-t];
        heading(2,j) = wave_dir(measured_time, vel);
        j = j+1;
        switcher = 1;
        plot(acc_est1(1,num)-time_corr,acc_est1(7,num),'rs',acc_est2(1,index(1))-time_corr,acc_est2(7,index(1)),'bs',...
            acc_est3(1,index(2))-time_corr,acc_est3(7,index(2)),'ms',acc_est4(1,index(3))-time_corr,acc_est4(7,index(3)),'gs');
    end

end
xlim([0 10])
set(gca, 'TickLabelInterpreter','latex')
xlabel('Time [s]','interpreter','latex')
ylabel('$a_z$ [$mm/s^2$]','interpreter','latex')
legend({'IMU\#1','IMU\#2','IMU\#3','IMU\#4'},'interpreter','latex','location','best','FontSize',7)
set(gca, 'YTick',[-500:100:500])
set(f,'PaperSize',[15, 8]);
print(f,'all_acc_angle_est_0degT1,5_new','-dpdf');
end

